@@ -352,11 +352,12 @@ class Catmaid:
         
         return dic
     
-    def get_neurons(self,neuron_id=None):
+    def get_neurons(self,neuron_id=None,scales=(1.,1.,1.)):
         """Return neuron data of a given list of neuron IDs.
         
         Args:
             neuron_id (int | array of int): list of neuron IDs.
+            scales (tuple (float)): define x, y, z scales
         
         Returns:
             list of neurons (tree.Tree)
@@ -389,9 +390,17 @@ class Catmaid:
                     if len(subdfcon)==0:
                         subdfcon = None
                         
+                # scale x, y and z
+                subdfneu["x"] = subdfneu["x"]*scales[0]
+                subdfneu["y"] = subdfneu["y"]*scales[1]
+                subdfneu["z"] = subdfneu["z"]*scales[2]
+                
                 dic[neuid] = trees.Tree.from_table(subdfneu,subdfcon,neuid,neuname)
                 
-        return dic
+        if len(neuronlst)==1: #only 1 tiem
+            return dic[neuid]
+        else:
+            return dic
     
     def to_csv(self,neuron_name="dfneu.csv",connector_name="dfcon.csv",neuron_id=None):
         """Export trees into dfneu.csv and dfcon.csv
