@@ -13,25 +13,25 @@ from PyAstronomy import pyasl
 
 from sklearn import metrics
 
-from genepy3d.util.geo import active_brownian_2d, active_brownian_3d, geo_len, norm, angle3points, vector2points, vector_axes_angles, angle2vectors
+from genepy3d.util.geo import active_brownian_2d, active_brownian_3d, geo_len, norm, angle3points, vector2points, vector_axes_angles, angle2vectors, l2 as distl2
 from genepy3d.util import plot as pl
 
 from matplotlib.colors import ListedColormap
 
 class Curve:
-    """Curve in 3D.
+    """Curve in 3D. Also work with 2D curve.
+    Check the documentation of each function to see if it supports 2D.
+    We assumme that curve is a parametric function g(t) = (x(t),y(t),z(t)).
+    If 2D points are passed then we consider as X and Y coordinates and set Z = 0.
     
     Attributes:
-        coors (array of float | tuple): list of 3d points.
+        coors (array | tuple): list of 3d points.
         
             - if ``array`` is given, then each column is the x, y, z coordinates of a specific point.
             - if ``tuple`` is given, then the items are the three arrays of x, y and z coordinates.
         
-        curve_id (int): curve ID.
-        curve_name (str): curve name.
-        
-    Note:
-        we assumme that curve is a parametric function g(t) = (x(t),y(t),z(t)).
+        curve_id (int): curve ID (optional).
+        curve_name (str): curve name (optional).
     
     """
     def __init__(self, coors, curve_id=None, curve_name=None):
@@ -41,6 +41,18 @@ class Curve:
         elif isinstance(coors,(tuple,list)):
             self.coors = np.array(coors).T
             
+        self.dim = 3 # default is 3D
+        
+        # check 2D input
+        if self.coors.shape[1]==2:
+            z = np.zeros(self.coors.shape[0])
+            newcoors = np.array([self.coors[:,0],self.coors[:,1],z]).T
+            self.coors = newcoors
+            self.dim = 2 # mark this curve is 2D
+            
+        
+        self.nb_of_points = self.coors.shape[0]
+            
         if curve_id is None:
             self.curve_id = 0 # default value
         else:
@@ -90,7 +102,7 @@ class Curve:
     
     @classmethod
     def from_text(cls,filepath):
-        """Read from text file (.txt, .xyz).
+        """Read from text-liked file (.txt, .xyz).
         
         Note:
             Assuming the first three columns correspond to x, y, z coordinates.
@@ -116,12 +128,14 @@ class Curve:
     def compute_derivative(self,deg,dt=1):
         """Derivative using np.gradient.
         
+        Support 2D and 3D. If in 2D, then return 0 for the derivative of Z.
+        
         Args:
             deg (int): derivative degree.
             dt (float): delta t.
             
         Returns:
-            array of float whose each row is the derivative in x, y and z at a specific point.
+            array of float where each row is the derivative in x, y and z at a specific point.
         
         """
         
@@ -151,6 +165,8 @@ class Curve:
     def compute_length(self):
         """Length of curve.
         
+        Support 2D and 3D.
+        
         Returns:
             float.
         
@@ -159,10 +175,10 @@ class Curve:
         return geo_len(self.coors)
     
     def compute_orientation(self,u=None):
-        """Orientation vectors at all points on the curve and angles w.r.t. vector u.
+        """Orientation vectors and angles w.r.t. a vector u for every point on the curve.
         
         Args:
-            u (array (float)): vector used to compute angles.
+            u (array (float)): vector used to compute angles. If None then return angles with three x, y and z axes.
         
         """
         
@@ -183,7 +199,7 @@ class Curve:
             vec = vector2points(a, b)
             vecs.append(vec)
             if u is None:
-                # angles wr.t. three axes x, y and z
+                # angles w.r.t. three axes x, y and z
                 angles.append(vector_axes_angles(vec))
             else:
                 angles.append(angle2vectors(vec,u))
@@ -193,6 +209,8 @@ class Curve:
     def compute_curvature(self):
         """Curvatures of curve.
         
+        Support 2D and 3D.
+        
         Returns:
             array of float.
         
@@ -215,6 +233,8 @@ class Curve:
     def compute_torsion(self):
         """Torsions of curve.
         
+        Only in 3D.
+        
         Returns:
             array of float.
         
@@ -237,6 +257,8 @@ class Curve:
     def compute_wiggliness(self):
         """Wiggliness of curve.
         
+        Support 2D and 3D.
+        
         Wiggliness = Length / Distance
         
         Returns:
@@ -256,20 +278,19 @@ class Curve:
     def compute_tortuosity(self):
         """Tortuosity of curve.
         
+        Same as compute_wiggliness()
+        
         Tortuosity = Length / Distance
         
         Returns:
             array of float.
-            
-        Note:
-            same function as compute_wiggliness().
         
         """
         
         return self.compute_wiggliness()
     
     def transform(self,phi=0,theta=0,psi=0,dx=0,dy=0,dz=0,zx=1,zy=1,zz=1):
-        """Make transformation.
+        """Apply rigid transformation to every point of the curve.
         
         Args:
             phi (float): rotation in x.
@@ -289,32 +310,32 @@ class Curve:
         
         return self.to_points().transform(phi,theta,psi,dx,dy,dz,zx,zy,zz).to_curve()
     
-    def compute_curviness(self):
-        """Curviness of curve.
+    # def compute_curviness(self):
+        # """Curviness of curve.
         
-        Curviness is computed from local maxima of curvatures.
+        # Curviness is computed from local maxima of curvatures.
         
-        Returns:
-            array of float.
+        # Returns:
+            # array of float.
         
-        """
+        # """
             
-        kappa = self.compute_curvature()
-        extid = argrelextrema(kappa,np.greater)[0] # get indices of local maxima of curvatures
+        # kappa = self.compute_curvature()
+        # extid = argrelextrema(kappa,np.greater)[0] # get indices of local maxima of curvatures
         
-        k = len(extid)
-        if k==0:
-            curviness = 0
-        else:
-            curviness = (1./(1.+abs(k)))*np.sum(kappa[extid])
+        # k = len(extid)
+        # if k==0:
+            # curviness = 0
+        # else:
+            # curviness = (1./(1.+abs(k)))*np.sum(kappa[extid])
                 
-        return curviness
+        # return curviness
     
     
     def _extract_plane(self,tor_thr,cur_thr,ext_thr):
-        """Compute planes and lines indices along curve based on torsions and curvatures.
+        """Compute indices where the curve is locally planar or linear based on torsions and curvatures.
         
-        This is the support function for ``scale_space()``.
+        Used in ``scale_space()``.
         
         Args:
             tor_thr (float): torsion threshold.
@@ -346,32 +367,59 @@ class Curve:
     def convolve_gaussian(self,sigma,mo="nearest",kerlen=4.0):
         """Get curve at a specific scale *sigma* by Gaussian convolution.
         
+        Support 2D and 3D.
+        
         Args:
-            sigma (float): scale.
+            sigma (float or list of floats): scale.
+            mo (str): mode used to treat the border effect (see gaussian_filter1d() for detail).
+            kerlen (float): specify the length of Gaussian kernel (see gaussian_filter1d() for detail).
             
         Returns:
             Curve.
         
         """
         
-        if sigma == 0:
-            return self
+        
+        if np.issubdtype(type(sigma),np.integer) | np.issubdtype(type(sigma),np.floating):
+            sigx, sigy, sigz = sigma, sigma, sigma
+        else:
+            try:
+                sigx, sigy, sigz = sigma[0], sigma[1], sigma[2]
+            except:
+                raise Exception('sigma must be a single number of a list')
+        
+        # if sigma == 0:
+        #     return self
         
         x, y, z = self.coors[:,0], self.coors[:,1], self.coors[:,2]
         
-        xs = gaussian_filter1d(x,sigma,mode=mo,truncate=kerlen)
-        ys = gaussian_filter1d(y,sigma,mode=mo,truncate=kerlen)
-        zs = gaussian_filter1d(z,sigma,mode=mo,truncate=kerlen)
+        if sigx == 0:
+            xs = x
+        else:
+            xs = gaussian_filter1d(x,sigx,mode=mo,truncate=kerlen)
+        
+        if sigy == 0:
+            ys = y
+        else:
+            ys = gaussian_filter1d(y,sigy,mode=mo,truncate=kerlen)
+        
+        if sigz == 0:
+            zs = z
+        else:
+            zs = gaussian_filter1d(z,sigz,mode=mo,truncate=kerlen)
         
         return Curve(np.array([xs,ys,zs]).T)
     
-    def resample(self,npoints=None,unit_length=None,spline_order=1):
+    def resample(self,npoints=None,unit_length=None,spline_order=1,return_interp_param=False):
         """Resample the curve using spline interpolation.
         
+        Support 2D and 3D.
+        
         Args:
             npoints (int): number of resampled points.
             unit_length (float): if not None, then ``npoints`` is calculated from it.
             spline_order (uint): 1 means linear interpolation.
+            return_interp_param (bool): if True then return interpolation parameters.
             
         Returns:
             Curve() resampled curve.
@@ -392,21 +440,47 @@ class Curve:
         else:
             # try to remove duplicates before interpolation
             _, uix = np.unique(self.coors,axis=0,return_index=True)
-            unique_coors = self.coors[np.sort(uix)]
-            
+            uix = np.sort(uix)
+            unique_coors = self.coors[uix]
+
             # interpolation
-            try:
-                coef, to = interpolate.splprep([unique_coors[:,0],unique_coors[:,1],unique_coors[:,2]],k=spline_order,s=0)
-            except:
-                raise Exception("Given coordinates can not be interpolated by spline.")
-            ti = np.linspace(0,1,n)
-            xi, yi, zi = interpolate.splev(ti,coef)
-            new_coors = np.array([xi,yi,zi]).T
+            if self.dim == 3:
+                try:
+                    coef, to = interpolate.splprep([unique_coors[:,0],unique_coors[:,1],unique_coors[:,2]],k=spline_order,s=0)
+                except:
+                    raise Exception("Given coordinates can not be interpolated by spline.")
+                ti = np.linspace(0,1,n)
+                xi, yi, zi = interpolate.splev(ti,coef)
+                new_coors = np.array([xi,yi,zi]).T
+            else: # 2D
+                try:
+                    coef, to = interpolate.splprep([unique_coors[:,0],unique_coors[:,1]],k=spline_order,s=0)
+                except:
+                    raise Exception("Given coordinates can not be interpolated by spline.")
+                ti = np.linspace(0,1,n)
+                xi, yi = interpolate.splev(ti,coef)
+                zi = [0. for _ in xi]
+                new_coors = np.array([xi,yi,zi]).T
         
-        return Curve(new_coors,self.curve_id,self.curve_name)
+        if return_interp_param==False:
+            return Curve(new_coors,self.curve_id,self.curve_name)
+        else:
+            try:
+                return (Curve(new_coors,self.curve_id,self.curve_name),to,coef,ti,uix)
+            except:
+                return Curve(new_coors,self.curve_id,self.curve_name)
     
-    def denoise(self):
-        """Automatically denoise a curve assuming Gaussian noise.
+    def denoise(self,return_std=False):
+        """Automatically denoise a curve assuming Gaussian noise. Noise variance acrooss the three dimensions is estimated using the beta-sigma method implemented in PyAstronomy [1]. 
+
+        [1] https://www.aanda.org/articles/aa/pdf/2018/01/aa30618-17.pdf 
+
+        args:
+            return_std (bool): return the estimated sigma as well (default: False)
+
+        return:
+            denoised curve (if return_std is False)
+            (denoised curve, estimated sigma) (if return_std is True)
         """
         
         with warnings.catch_warnings():
@@ -461,18 +535,21 @@ class Curve:
         else:
             crvd = self
         
-        return crvd
+        if return_std == False:
+            return crvd
+        else:
+            return (crvd,estsig)
     
     def scale_space(self,scale_range,features={'curvature','torsion','ridge','valley','planeline','line'},eps_kappa=0.01,eps_tau=0.01,eps_seg=10,mo="nearest",kerlen=4.0):
-        """Compute features of curve in scale space.
+        """Compute features of a curve across scales, i.e. a scale space of gaussian convolution.
         
         Can compute various features at one time.
         
         Args:
-            scale_range (array of float): range of scales.
+            scale_range (array of float): range of scales (sigma of gaussian convolutions).
             features (dic): list of features (detail see below).
-            eps_kappa (float): curvature threshold.
-            eps_tau (float): torsion threshold.
+            eps_kappa (float): curvature threshold. (for planeline feature)
+            eps_tau (float): torsion threshold. (for line and planeline feature)
             eps_seg (float): segment length threshold (in number of points).
             
         Returns:
@@ -715,7 +792,7 @@ class Curve:
         
         # setting minimal segment length
         if eps_seg_len is None:
-            eps_seg = int(np.round(self.coors.shape[0]*0.1)) # 10% of nb. of points
+            eps_seg = int(np.round(self.coors.shape[0]*0.05)) # 5% of nb. of points
         else:
             # converting eps_seg_len in nb. of points
             if self.compute_length() <= eps_seg_len:
@@ -738,7 +815,10 @@ class Curve:
         
         # setting minimal curve length
         if eps_crv_len is None:
-            eps_crv = self.compute_length()*0.01 # 1% of curve length
+            if eps_seg_len is not None:
+                eps_crv = eps_seg_len
+            else:
+                eps_crv = self.compute_length()*0.01 # 1% of curve length
         else:
             eps_crv = eps_crv_len
         
@@ -810,9 +890,9 @@ class Curve:
         
     def plot_intrinsicdim(self,ax,projection='3d',scales=(1.,1.,1.),
                           root_args={"c":"blue","s":100},
-                          dim1d_args={"c":"magenta","lw":3,"alpha":1.},
-                          dim2d_args={"c":"yellow","lw":5,"alpha":0.7},
-                          dim3d_args={"c":"green","lw":5,"alpha":0.7},
+                          dim1d_args={"c":"magenta","lw":1,"alpha":1.},
+                          dim2d_args={"c":"yellow","lw":3,"alpha":0.7},
+                          dim3d_args={"c":"green","lw":3,"alpha":0.7},
                           plane_color="yellow",
                           overrided_curve=None):
         """Display curve superimposed by estimated intrinsic dimensions.
@@ -877,8 +957,90 @@ class Curve:
             ax.set_ylim(param['ymin'],param['ymax'])
             ax.set_zlim(param['zmin'],param['zmax'])
             
-    def compute_local_3d_scale(self,r_lst,eps_seg_len=None,eps_crv_len=None,sig_step=1,eps_kappa=0.01,eps_tau=0.01,return_dim_results=False):
-        """Computing local 3d scale of curve.
+    
+    def compute_local_3d_scale_sigma(self,sig_lst,delta_sig=0,sig_step=1,eps_seg_len=None,eps_crv_len=None,eps_kappa=0.01,eps_tau=0.01,return_dim_results=False):
+        """Computing local 3d scale of curve from the list of sigma i.e stdev of Gaussian function.
+        
+        Local 3D scale: scale at which point transforms from 3D to 2D/1D.
+        
+        Args:
+            sig_lst (list (float)): list of sigma values.
+            delta_sig (float): width of the searching interval.
+            sig_step (int|float): scale step within the searching region.
+            eps_seg_len (float): segment length threshold.
+            eps_crv_len (float): curve length threshold.
+            eps_kappa (float): curvature threshold.
+            eps_tau (float): torsion threshold.
+            return_dim_results (bool): if True, then return intrinsic dim estimation of r_lst
+            
+        Returns:
+            list of local 3d scales for every points on curve.
+            
+        """
+        
+        npnt = self.coors.shape[0]
+        nrad = len(sig_lst)
+        counter = np.zeros(npnt)
+        maxlen = np.ones(npnt)*-1.
+        firstid = np.ones(npnt)*(nrad-1)
+        
+        if return_dim_results==True:
+            dim_results = []
+            
+        # default delta sigma
+        if delta_sig is None:
+            delta_sig = 3 * sig_step
+        
+        # intrinsic dim estimation
+        for ir in range(nrad):
+                
+            sc = sig_lst[ir]
+            res = self.decompose_intrinsicdim(sc,delta_sig,
+                                             eps_seg_len,eps_crv_len,sig_step,
+                                             eps_kappa,eps_tau)
+            
+            if return_dim_results==True:
+                dim_results.append(res)
+            
+            dimid = []
+            for item in res['planeline_pred']:
+                dimid = dimid + list(range(item[0],item[1]+1))
+            
+            countedid = np.argwhere(counter>0).flatten()
+            intersectedid = np.intersect1d(countedid,dimid)
+            diffid = np.setdiff1d(countedid,dimid)
+            if len(intersectedid)>0:
+                counter[intersectedid] = counter[intersectedid] + 1.
+            if len(diffid)>0:
+                checkid = np.argwhere(counter[diffid]>maxlen[diffid]).flatten()
+                if len(checkid)>0:
+                    updateid = diffid[checkid]
+                    maxlen[updateid] = counter[updateid]
+                    firstid[updateid] = ir - counter[updateid]
+                    counter[updateid] = 0. # reset counter
+
+            uncountedid = np.argwhere(counter==0).flatten()
+            intersectedid = np.intersect1d(uncountedid,dimid)
+            if len(intersectedid)>0:
+                counter[intersectedid] = 1.
+                
+        # remove peaks
+        nonvalidid = np.argwhere(maxlen==1.).flatten()
+        firstid[nonvalidid] = nrad-1
+
+        # when the point has 2D/1D until the end of radius
+        finalcheckid = np.argwhere((counter>maxlen)&(counter!=0)).flatten()
+        if len(finalcheckid)>0:
+            firstid[finalcheckid] = nrad - counter[finalcheckid]
+        
+        if return_dim_results==False:
+            return list(sig_lst[firstid.astype(np.uint)])
+        else:
+            return (list(sig_lst[firstid.astype(np.uint)]),dim_results)
+    
+    
+    def compute_local_3d_scale_radius(self,r_lst,eps_seg_len=None,eps_crv_len=None,sig_step=1,eps_kappa=0.01,eps_tau=0.01,return_dim_results=False):
+        """Computing local 3d scale of curve from the list of radius of curvatures.
         
         Local 3D scale: scale at which point transforms from 3D to 2D/1D.
         
@@ -902,7 +1064,93 @@ class Curve:
         maxlen = np.ones(npnt)*-1.
         firstid = np.ones(npnt)*(nrad-1)
         
-        rstbl = RadiusScaleTable(self,max(r_lst))
+        rstbl = RadiusScaleTable(self,max(r_lst)+1)
+        
+        if return_dim_results==True:
+            dim_results = []
+            
+        
+        # intrinsic dim estimation
+        for ir in range(nrad):
+            try:
+                sc, delta_sig = rstbl.compute_scales_from_radius(r_lst[ir])
+            except:
+                raise Exception("failed at ir={}".format(ir))
+            
+            res = self.decompose_intrinsicdim(sc,delta_sig,
+                                             eps_seg_len,eps_crv_len,sig_step,
+                                             eps_kappa,eps_tau)
+            
+            if return_dim_results==True:
+                dim_results.append(res)
+            
+            dimid = []
+            for item in res['planeline_pred']:
+                dimid = dimid + list(range(item[0],item[1]+1))
+            
+            countedid = np.argwhere(counter>0).flatten()
+            intersectedid = np.intersect1d(countedid,dimid)
+            diffid = np.setdiff1d(countedid,dimid)
+            if len(intersectedid)>0:
+                counter[intersectedid] = counter[intersectedid] + 1.
+            if len(diffid)>0:
+                checkid = np.argwhere(counter[diffid]>maxlen[diffid]).flatten()
+                if len(checkid)>0:
+                    updateid = diffid[checkid]
+                    maxlen[updateid] = counter[updateid]
+                    firstid[updateid] = ir - counter[updateid]
+                    counter[updateid] = 0. # reset counter
+
+            uncountedid = np.argwhere(counter==0).flatten()
+            intersectedid = np.intersect1d(uncountedid,dimid)
+            if len(intersectedid)>0:
+                counter[intersectedid] = 1.
+                
+        # remove peaks
+        nonvalidid = np.argwhere(maxlen==1.).flatten()
+        firstid[nonvalidid] = nrad-1
+
+        # when the point has 2D/1D until the end of radius
+        finalcheckid = np.argwhere((counter>maxlen)&(counter!=0)).flatten()
+        if len(finalcheckid)>0:
+            firstid[finalcheckid] = nrad - counter[finalcheckid]
+        
+        if return_dim_results==False:
+            return list(r_lst[firstid.astype(np.uint)])
+        else:
+            return (list(r_lst[firstid.astype(np.uint)]),dim_results)
+    
+    
+    def compute_local_3d_scale(self,r_lst,eps_seg_len=None,eps_crv_len=None,sig_step=1,eps_kappa=0.01,eps_tau=0.01,return_dim_results=False):
+        """Computing local 3d scale of curve. [1]
+        
+        Local 3D scale: scale at which point transforms from 3D to 2D/1D.
+
+        [1] https://doi.org/10.1101/2020.06.01.127035
+        
+        This function is DEPRECATED. Use compute_local_3d_scale_radius().
+        
+        Args:
+            r_lst (list (float)): list of radii of curvatures.
+            eps_seg_len (float): segment length threshold.
+            eps_crv_len (float): curve length threshold.
+            sig_step (int): scale step within the searching region.
+            eps_kappa (float): curvature threshold.
+            eps_tau (float): torsion threshold.
+            return_dim_results (bool): if True, then return intrinsic dim estimation of r_lst
+            
+        Returns:
+            list of local 3d scales for every points on curve.
+            
+        """
+        
+        npnt = self.coors.shape[0]
+        nrad = len(r_lst)
+        counter = np.zeros(npnt)
+        maxlen = np.ones(npnt)*-1.
+        firstid = np.ones(npnt)*(nrad-1)
+        
+        rstbl = RadiusScaleTable(self,max(r_lst)+1)
         
         if return_dim_results==True:
             dim_results = []
@@ -1057,7 +1305,7 @@ class Curve:
                 idx = idx + 1
         return newbreakid
     
-    def main_turns(self,nbscales=None,search_step=5,eps_kappa=None,ridgelength_thr=0.1,angle_thr=20):
+    def main_turns(self,nbscales=None,search_step=5,eps_kappa=None,ridgelength_thr=0.1,angle_thr=20,min_dist=None):
         """Compute the main turns of curve.
         
         Args:
@@ -1066,6 +1314,7 @@ class Curve:
             eps_kappa (float): curvature threshold.
             ridgelength_thr (float): ridge length threshold (percentage compared to the longest ridge).
             angle_thr (float): angle threshold in degree.
+            min_dist (float): minimal distance between two main turns.
             
         Returns:
             indices of main turns.
@@ -1077,7 +1326,7 @@ class Curve:
         # extid = argrelextrema(crvr.compute_curvature(),np.greater)[0]
         
         if nbscales is None:
-            ref_r = self.compute_length()/4.
+            ref_r = self.compute_length()/3.
             rstbl = RadiusScaleTable(self,ref_r)
             _nbscales, _ = rstbl.compute_scales_from_radius()
         else:
@@ -1095,7 +1344,11 @@ class Curve:
         
         # identify ridges in curvature scale space
         ridge_lst, length_lst, warning_lst = [],[],[]
-        start_idx = np.argwhere((data['ridge'][0,:]!=0)&(data['curvature'][0,:]>kappa_thr)).flatten()
+        
+        try:
+            start_idx = np.argwhere((data['ridge'][0,:]!=0)&(data['curvature'][0,:]>kappa_thr)).flatten()
+        except:
+            return [] # there is no data for ridge and curvature
         
         for idx in start_idx:
             ridge,warning = self._find_ridge(idx,data['ridge'],search_step)
@@ -1106,7 +1359,7 @@ class Curve:
         
         
         if (len(length_arr)==0):
-            return None
+            return []
     
         # select ridges based on ridge length
         p = max(length_arr)*ridgelength_thr
@@ -1134,6 +1387,18 @@ class Curve:
                 newbreakid = np.delete(newbreakid,idx)
                 
         selected_idx = newbreakid[1:-1]
+        
+        # remove turns whose distance is too small
+        if (min_dist != None) & (len(selected_idx)>1):
+            check_idx = 0
+            while(check_idx<len(selected_idx)-1):
+                ix1, ix2 = selected_idx[check_idx], selected_idx[check_idx+1]
+                coors = self.coors[ix1:ix2+1]
+                if(geo_len(coors)<=min_dist):
+                    selected_idx = np.delete(selected_idx,check_idx+1)
+                else:
+                    check_idx += 1
+        
         # ignored_idx = np.setdiff1d(extid,selected_idx)
         
         # self.ppt = {} # main turns results
@@ -1246,11 +1511,12 @@ class SimuIntrinsic:
     
     """
     
-    def __init__(self,seg_lbl=[0,1,2],npoints=1000,sigma=1.,random_state=None,params={},max_seg=None,nb_seg=None,remove_zero_replica=True):
+    def __init__(self,seg_lbl=[0,1,2],npoints=1000,sigma=1.,random_state=None,params={},max_seg=None,nb_seg=None,remove_zero_replica=True,spline_order=1):
         
         self.npoints = npoints
         self.sigma = sigma
         self.random_state = random_state
+        self.spline_order = spline_order
         
         # generate segment labels
         if nb_seg is not None:
@@ -1459,7 +1725,7 @@ class SimuIntrinsic:
                 P = P * 1e6 # to micron
                 
                 # interpolation
-                coef, to = interpolate.splprep([P[:,0],P[:,1],P[:,2]],k=1,s=0)
+                coef, to = interpolate.splprep([P[:,0],P[:,1],P[:,2]],k=self.spline_order,s=0)
                 ti = np.linspace(0,1,1000)
                 xi, yi, zi = interpolate.splev(ti,coef)
                 Pi = np.array([xi,yi,zi]).T
@@ -1486,7 +1752,7 @@ class SimuIntrinsic:
                 P = np.append(P,np.zeros((P.shape[0],1)),axis=1) # add z coordinates
                 
                 # interpolation
-                coef, to = interpolate.splprep([P[:,0],P[:,1],P[:,2]],k=1,s=0)
+                coef, to = interpolate.splprep([P[:,0],P[:,1],P[:,2]],k=self.spline_order,s=0)
                 ti = np.linspace(0,1,1000)
                 xi, yi, zi = interpolate.splev(ti,coef)
                 Pi = np.array([xi,yi,zi]).T
@@ -1517,10 +1783,10 @@ class SimuIntrinsic:
 #                alpha = (((-np.pi/4.) - (-np.pi/3.)) * np.random.random(1) + (-np.pi/3.))[0]
                 beta = (((2*np.pi) - (0.)) * np.random.random(1) + (0.))[0]
 #                print(beta)
-                if (iln%2)!=0:
-                    phi, theta, psi = 0.,alpha,beta
-                else:
+                if (iln%2)==0:
                     phi, theta, psi = alpha,0.,beta
+                else:
+                    phi, theta, psi = 0.,alpha,beta
                 
                 Pt = Points(P).transform(phi,theta,psi,0.,0.,0.).coors
                 
@@ -1547,7 +1813,7 @@ class SimuIntrinsic:
             
             # interpolation
             P = tmp[iseg]
-            coef, to = interpolate.splprep([P[:,0],P[:,1],P[:,2]],k=1,s=0)
+            coef, to = interpolate.splprep([P[:,0],P[:,1],P[:,2]],k=self.spline_order,s=0)
             ti = np.linspace(0,1,npoints_seg)
             xi, yi, zi = interpolate.splev(ti,coef)
             Pi = np.array([xi,yi,zi]).T
@@ -1991,7 +2257,7 @@ class RadiusScaleMap:
         save_circles (bool): if True, then save the scaled circle for each scale.
         
     Note:
-        This class is deprecated.
+        This class is DEPRECATED.
     
     """
     
@@ -2124,9 +2390,10 @@ class RadiusScaleMap:
         return (sig_c,delta_c,rmask)
 
 def decompose_intrinsicdim_sequential(p,eps_line,eps_plane,eps_seg):
-    """Decompose sequentially intrinsic segments with priority line to plane to 3d
+    """Decompose sequentially intrinsic segments with priority line to plane to 3d.
     
-    from paper of Jianyu Yang: https://www.sciencedirect.com/science/article/pii/S1047320316300438
+    from paper of Jianyu Yang: https://www.sciencedirect.com/science/article/pii/S1047320316300438.
+    This method was selected to compare with our decompose_intrinsicdim().
     
     Args:
         p (array of float): list of 3D points on curve.
@@ -2232,7 +2499,10 @@ def decompose_intrinsicdim_sequential(p,eps_line,eps_plane,eps_seg):
 def align(target, ref):
     """Align target curve to reference curve.
     
-    This func is used for aligning curves before using EMD. But it should be improved to be used wisely.
+    This func is used to align two curves that minimize their spatial differences.
+    
+    The func is useful as a preparation before computing the distance between the two curves.
+    E.g. we provided emd() in Points class to compute the Earth Mover's Distance between two point clouds.
     
     Args:
         target (Curve): target curve.
